@@ -1,14 +1,14 @@
-/* @(#)longnames.c	1.52 09/10/16 Copyright 1993, 1995, 2001-2009 J. Schilling */
+/* @(#)longnames.c	1.57 13/10/09 Copyright 1993, 1995, 2001-2013 J. Schilling */
 #include <schily/mconfig.h>
 #ifndef lint
 static	UConst char sccsid[] =
-	"@(#)longnames.c	1.52 09/10/16 Copyright 1993, 1995, 2001-2009 J. Schilling";
+	"@(#)longnames.c	1.57 13/10/09 Copyright 1993, 1995, 2001-2013 J. Schilling";
 #endif
 /*
  *	Handle filenames that cannot fit into a single
  *	string of 100 charecters
  *
- *	Copyright (c) 1993, 1995, 2001-2009 J. Schilling
+ *	Copyright (c) 1993, 1995, 2001-2013 J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -17,6 +17,8 @@ static	UConst char sccsid[] =
  * with the License.
  *
  * See the file CDDL.Schily.txt in this distribution for details.
+ * A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file CDDL.Schily.txt from this distribution.
@@ -101,7 +103,17 @@ error("low: %d:%s high: %d:'%c',%s\n",
 	while (--high >= low)
 		if (*high == '/')
 			break;
-	if (high < low) {
+	/*
+	 * We cannot split if we did either find no '/' (high < low), or if
+	 * there is no name past the '/' we found (this is when a name ends
+	 * on a '/' and there is no way to split off a suffix with a length
+	 * that is <= 100 chars.
+	 * Star and gtar would be happy with a zero length suffix, but
+	 * traditional tar implementations ignore their own logical EOF
+	 * definition that requires two blocks full of zeroes and stop on
+	 * a null character at dbuf.t_name[0] already.
+	 */
+	if (high < low || high[1] == '\0') {
 		if ((props.pr_nflags & PR_LONG_NAMES) == 0) {
 			if (!errhidden(E_NAMETOOLONG, name)) {
 				if (!errwarnonly(E_NAMETOOLONG, name))
@@ -153,9 +165,9 @@ name_to_tcb(info, ptb)
 
 	if ((namelen+add) <= props.pr_maxsname) {	/* Fits in shortname */
 		if (add)
-			strcatl(ptb->dbuf.t_name, name, "/", (char *)NULL);
+			strcatl(ptb->ndbuf.t_name, name, "/", (char *)NULL);
 		else
-			strcpy(ptb->dbuf.t_name, name);
+			strcpy(ptb->ndbuf.t_name, name);
 		return (TRUE);
 	}
 
@@ -196,16 +208,17 @@ name_to_tcb(info, ptb)
 	 * Do actual splitting based on split name pointer 'np'.
 	 */
 	if (add)
-		strcatl(ptb->dbuf.t_name, &np[1], "/", (char *)NULL);
+		strcatl(ptb->ndbuf.t_name, &np[1], "/", (char *)NULL);
 	else
-		strcpy(ptb->dbuf.t_name, &np[1]);
+		strcpy(ptb->ndbuf.t_name, &np[1]);
 	strncpy(ptb->dbuf.t_prefix, name, np - name);
 	info->f_flags |= F_SPLIT_NAME;
 	return (TRUE);
 }
 
 /*
- * This function is only called by tcb_to_info().
+ * This function is only called by tcb_to_info() with
+ * dbuf.t_name[100], dbuf.t_linkname[100] and dbuf.t_prefix[155] set to '\0'.
  * If we ever decide to call it from somewhere else check if the linkname
  * kludge for 100 char linknames does not make problems.
  */
@@ -221,14 +234,14 @@ tcb_to_name(ptb, info)
 	if (info->f_flags & F_HAS_NAME)
 		return;
 
-	if ((info->f_flags & F_LONGLINK) == 0 &&	/* name from 'K' head*/
-	    (info->f_xflags & XF_LINKPATH) == 0) {	/* name from 'x' head*/
+	if ((info->f_flags & F_LONGLINK) == 0 &&	/* name from 'K' head */
+	    (info->f_xflags & XF_LINKPATH) == 0) {	/* name from 'x' head */
 
 		/*
 		 * Our caller has set ptb->dbuf.t_linkname[NAMSIZ] to '\0'
 		 * if the link name len is exactly 100 chars.
 		 */
-		strcpy(info->f_lname, ptb->dbuf.t_linkname);
+		strcpy(info->f_lname, ptb->ndbuf.t_linkname);
 	}
 
 	/*
@@ -243,11 +256,11 @@ tcb_to_name(ptb, info)
 		return;
 
 	if (props.pr_nflags & PR_POSIX_SPLIT) {
-		strcatl(info->f_name, ptb->dbuf.t_prefix,
-					*ptb->dbuf.t_prefix?"/":"",
-					ptb->dbuf.t_name, (char *)NULL);
+		strcatl(info->f_name, ptb->ndbuf.t_prefix,
+					*ptb->ndbuf.t_prefix?"/":"",
+					ptb->ndbuf.t_name, (char *)NULL);
 	} else {
-		strcpy(info->f_name, ptb->dbuf.t_name);
+		strcpy(info->f_name, ptb->ndbuf.t_name);
 	}
 }
 
@@ -301,14 +314,18 @@ tcb_to_longname(ptb, info)
 		 * We do not know the name here,
 		 * we only have the short ptb->dbuf.t_name
 		 */
-/*		if (!errhidden(E_NAMETOOLONG, name)) {*/
-/*			if (!errwarnonly(E_NAMETOOLONG, name))*/
+#ifdef	__can_we__
+		if (!errhidden(E_NAMETOOLONG, name)) {
+			if (!errwarnonly(E_NAMETOOLONG, name))
+#endif
 				xstats.s_toolong++;
 			errmsgno(EX_BAD,
 			"Long name too long (%lld) ignored.\n",
 							(Llong)info->f_size);
-/*			(void) errabort(E_NAMETOOLONG, name, TRUE);*/
-/*		}*/
+#ifdef	__can_we__
+			(void) errabort(E_NAMETOOLONG, name, TRUE);
+		}
+#endif
 		void_file(info);
 		return (get_tcb(ptb));
 	}
