@@ -1,8 +1,8 @@
-/* @(#)xutimes.h	1.2 03/06/12 Copyright 1996, 2003 J. Schilling */
+/* @(#)xutimes.h	1.3 13/10/02 Copyright 1996, 2013 J. Schilling */
 /*
  *	Prototypes for xutimes users
  *
- *	Copyright (c) 1996, 2003 J. Schilling
+ *	Copyright (c) 1996, 2013 J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -11,6 +11,8 @@
  * with the License.
  *
  * See the file CDDL.Schily.txt in this distribution for details.
+ * A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file CDDL.Schily.txt from this distribution.
@@ -19,4 +21,4 @@
 /*
  * star_unix.c
  */
-extern	int	xutimes		__PR((char *name, struct timeval *tp));
+extern	int	xutimes		__PR((char *name, struct timespec *tp));
