@@ -1,10 +1,11 @@
-/* @(#)tartest.c	1.12 06/10/31 Copyright 2002-2006 J. Schilling */
+/* @(#)tartest.c	1.19 13/10/09 Copyright 2002-2013 J. Schilling */
+#include <schily/mconfig.h>
 #ifndef lint
-static	char sccsid[] =
-	"@(#)tartest.c	1.12 06/10/31 Copyright 2002-2006 J. Schilling";
+static	UConst char sccsid[] =
+	"@(#)tartest.c	1.19 13/10/09 Copyright 2002-2013 J. Schilling";
 #endif
 /*
- *	Copyright (c) 2002-2006 J. Schilling
+ *	Copyright (c) 2002-2013 J. Schilling
  */
 /*
  * The contents of this file are subject to the terms of the
@@ -13,13 +14,14 @@ static	char sccsid[] =
  * with the License.
  *
  * See the file CDDL.Schily.txt in this distribution for details.
+ * A copy of the CDDL is also available via the Internet at
+ * http://www.opensource.org/licenses/cddl1.txt
  *
  * When distributing Covered Code, include this CDDL HEADER in each
  * file and include the License file CDDL.Schily.txt from this distribution.
  */
 
-#include <schily/mconfig.h>
-#include <stdio.h>
+#include <schily/stdio.h>
 #include <schily/stdlib.h>
 
 #include "star.h"
@@ -28,10 +30,8 @@ static	char sccsid[] =
 #include <schily/getargs.h>
 #include <schily/schily.h>
 
-#ifdef	NEED_O_BINARY
-#include <io.h>					/* for setmode() prototype */
 #include <schily/fcntl.h>			/* O_BINARY */
-#endif
+#include <schily/io.h>				/* for setmode() prototype */
 
 LOCAL	void	usage		__PR((int ret));
 EXPORT	int	main		__PR((int ac, char *av[]));
@@ -57,6 +57,7 @@ usage(ret)
 	error("Usage:\t%s [options] < file\n", get_progname());
 	error("Options:\n");
 	error("\t-help\t\tprint this help\n");
+	error("\t-version\tPrint version number.\n");
 	error("\t-v\t\tprint all filenames during verification\n");
 	error("\n%s checks stdin fore compliance with the POSIX.1-1990 TAR standard\n", get_progname());
 	exit(ret);
@@ -71,28 +72,30 @@ main(ac, av)
 	int		cac = ac;
 	char	*const *cav = av;
 	BOOL		help = FALSE;
+	BOOL		prversion = FALSE;
 
 	save_args(ac, av);
 	cac--;
 	cav++;
-	if (getallargs(&cac, &cav, "help,h,v", &help, &help, &verbose) < 0) {
+	if (getallargs(&cac, &cav, "help,h,version,v", &help, &help,
+						&prversion, &verbose) < 0) {
 		errmsgno(EX_BAD, "Bad Option: '%s'.\n", cav[0]);
 		usage(EX_BAD);
 	}
 	if (help)
 		usage(0);
 
-	printf("tartest %s (%s-%s-%s)\n\n", "1.12",
+	printf("tartest %s (%s-%s-%s)\n\n", "1.19",
 					HOST_CPU, HOST_VENDOR, HOST_OS);
-	printf("Copyright (C) 2002 Jrg Schilling\n");
+	printf("Copyright (C) 2002-2013 Jrg Schilling\n");
 	printf("This is free software; see the source for copying conditions.  There is NO\n");
 	printf("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
+	if (prversion)
+		exit(0);
 
 	printf("\nTesting for POSIX.1-1990 TAR compliance...\n");
 
-#ifdef	NEED_O_BINARY
 	setmode(fileno(stdin), O_BINARY);
-#endif
 
 	if (!doit(stdin)) {
 		printf(">>> Archive is not POSIX.1-1990 TAR standard compliant.\n");
@@ -150,7 +153,8 @@ doit(f)
 			}
 			printf("Found 2nd EOF block at %lld\n", blockno);
 			return (ret);
-		} if (checks != hsum) {
+		}
+		if (checks != hsum) {
 			printf("Bad Checksum %0llo != %0llo at block %lld\n",
 							checks, hsum, blockno);
 			signedcksum = TRUE;
@@ -253,17 +257,21 @@ checkhdr(ptb)
 	int	errs = 0;
 	Ullong	ll;
 
+	if (ptb->ustar_dbuf.t_name[  0] == '\0') {
+		printf("Warning: t_name[  0] is a null character.\n");
+		errs++;
+	}
 	if (ptb->ustar_dbuf.t_name[  0] != '\0' &&
 	    ptb->ustar_dbuf.t_name[ 99] != '\0' &&
 	    /* LINTED */
-	    ptb->ustar_dbuf.t_name[100] == '\0') {
+	    ptb->ndbuf.t_name[100] == '\0') {
 		printf("Warning: t_name[100] is a null character.\n");
 		errs++;
 	}
 	if (ptb->ustar_dbuf.t_linkname[  0] != '\0' &&
 	    ptb->ustar_dbuf.t_linkname[ 99] != '\0' &&
 	    /* LINTED */
-	    ptb->ustar_dbuf.t_linkname[100] == '\0') {
+	    ptb->ndbuf.t_linkname[100] == '\0') {
 		printf("Warning: t_linkname[100] is a null character.\n");
 		errs++;
 	}
@@ -348,7 +356,9 @@ checkoctal(ptr, len, text)
 	char	cs[4];
 
 	for (i = 0; i < len; i++) {
-/*		error("%d '%c'\n", i, ptr[i]);*/
+#ifdef	CHECKOCTAL_DEBUG
+		error("%d '%c'\n", i, ptr[i]);
+#endif
 
 #ifdef		END_ALL_THESAME
 		if (endoff > 0 && ptr[i] != endc) {
@@ -552,7 +562,7 @@ checkvers(ptr)
  * This is the debug version that stops at "len" size to be safe against
  * field overflow.
  */
-EXPORT void /*char **/
+EXPORT void
 stolli(s, ull, len)
 	register char	*s;
 		Ullong	*ull;
@@ -572,7 +582,9 @@ stolli(s, ull, len)
 		if (--len < 0)
 			break;
 		c = *s++;
-/*		error("'%c'\n", c);*/
+#ifdef	STOLLI_DEBUG
+		error("'%c'\n", c);
+#endif
 		if (isoctal(c))
 			t = c - '0';
 		else
@@ -581,8 +593,9 @@ stolli(s, ull, len)
 		ret += t;
 	}
 	*ull = ret;
-/*	error("len: %d\n", len);*/
-	/*return(s);*/
+#ifdef	STOLLI_DEBUG
+	error("len: %d\n", len);
+#endif
 }
 
 /*
